/* Copyright (C) 2005-2019 Shugo Maeda <shugo@ruby-lang.org> and Charlie Savage <cfis@savagexi.com>
   Please see the LICENSE file for copyright and distribution information */

#ifndef __BF_PROFILE_H__
#define __BF_PROFILE_H__

#include "bf_measurement.h"
#include "bf_thread.h"
#include "blackfire.h"

extern VALUE cProfile;

#define DEFAULT_TIMESPAN_THRESHOLD 10 // ms

typedef struct prof_profile_t
{
    /* internal data */
    prof_measurer_t *measurer;
    st_table *threads_tbl;
    st_table *exclude_threads_tbl;
    st_table *include_threads_tbl;
    st_table *exclude_methods_tbl;
    thread_data_t *last_thread_data;
    struct four_doubles_t measurement_at_pause_resume;
    bool allow_exceptions;

    // timespan related
    bool flag_timespan;
    unsigned timespan_threshold; // in ms
    VALUE timespan_rules;
    VALUE timespan_entries;

    // fn-args rules are passed as an Ruby hash to the C extension
    VALUE fnarg_rules;

    // the measurement done in first enable()
    struct four_doubles_t initial_measurement;

    /* outputs  */
    VALUE running;
    VALUE paused;
    VALUE tracepoints;
} prof_profile_t;

void rp_init_profile(void);
prof_profile_t *prof_get_profile(VALUE self);

#endif //__BF_PROFILE_H__
