/* Copyright (C) 2005-2019 Shugo Maeda <shugo@ruby-lang.org> and Charlie Savage <cfis@savagexi.com>
   Please see the LICENSE file for copyright and distribution information */

#ifndef __BF_STACK__
#define __BF_STACK__

#include "bf_call_tree.h"
#include "blackfire.h"

/* Temporary object that maintains profiling information
   for active methods.  They are created and destroyed
   as the program moves up and down its stack. */
typedef struct prof_frame_t
{
    /* Caching prof_method_t values significantly
       increases performance. */
    prof_call_tree_t *call_tree;

    VALUE source_file;
    unsigned int source_line;

    struct four_doubles_t start_time;
    struct four_doubles_t switch_time; /* Time at switch to different thread */
    struct four_doubles_t wait_time;
    struct four_doubles_t child_time;
    struct four_doubles_t pause_time; // Time pause() was initiated
    struct four_doubles_t dead_time;  // Time to ignore (i.e. total amount of time between pause/resume blocks)
} prof_frame_t;

#define prof_frame_is_paused(f) (f->pause_time.wt >= 0 && f->pause_time.al >= 0 && f->pause_time.mm >= 0 && f->pause_time.pr >= 0)
#define prof_frame_is_unpaused(f) (f->pause_time.wt < 0 && f->pause_time.al < 0 && f->pause_time.mm < 0 && f->pause_time.pr < 0)

void prof_frame_pause(prof_frame_t *, struct four_doubles_t current_measurement);
void prof_frame_unpause(prof_frame_t *, struct four_doubles_t current_measurement);

/* Current stack of active methods.*/
typedef struct prof_stack_t
{
    prof_frame_t *start;
    prof_frame_t *end;
    prof_frame_t *ptr;
} prof_stack_t;

prof_stack_t *prof_stack_create(void);
void prof_stack_free(prof_stack_t *stack);

prof_frame_t *prof_frame_current(prof_stack_t *stack);
prof_frame_t *prof_frame_push(prof_stack_t *stack, prof_call_tree_t *call_tree, struct four_doubles_t measurement, bool paused);
prof_frame_t *prof_frame_unshift(prof_stack_t *stack, prof_call_tree_t *parent_call_tree, prof_call_tree_t *call_tree, struct four_doubles_t measurement);
prof_frame_t *prof_frame_pop(prof_stack_t *stack, struct four_doubles_t measurement, prof_measurer_t *measurer);
prof_method_t *prof_find_method(prof_stack_t *stack, VALUE source_file, int source_line);

#endif //__BF_STACK__
