module Blackfire

  class FlatPrinter < AbstractPrinter
    include PrinterHelper
    attr_accessor :query_current_order, :time_to_remove

    def increment_query_current_order(call_tree)
      original_method = call_tree.target.full_name.split("#")[1]
      return if QueryTrace.exec_methods.nil?

      @query_current_order += 1 if QueryTrace.exec_methods.include? original_method
    end

    def new_trace(key, tree_item)
      return BlackfireTrace.new(key,
                  tree_item.called,
                  tree_item.wall_time_total_time * MULTIPLIER_FOR_TIME,
                  tree_item.process_total_time * MULTIPLIER_FOR_TIME,
                  tree_item.allocation_total_time,
                  tree_item.memory_total_time * MULTIPLIER_FOR_MEMORY)
    end

    def print_methods(thread)
      visitor = CallTreeVisitor.new(thread.call_tree)
      first = true
      metrics = ""
      @query_current_order = -1
      @time_to_remove = nil
      root_method = ""

      traces = {}
      visitor.visit do |call_tree_item, event|
        case event
        when :enter
          if first
            root_method = call_tree_item.target.full_name
            @output << "Graph-Root-Id: #{root_method}\n"
            first = false
          end

          increment_query_current_order(call_tree_item)

          trace_key = build_target_name(call_tree_item, @query_current_order)

          if traces.key?(trace_key)
            new_trace = new_trace(trace_key, call_tree_item)
            traces[trace_key] += new_trace
          else
            traces[trace_key] = new_trace(trace_key, call_tree_item)
          end
        end
      end

      traces.each do | key, trace |
        metrics << format(
            "%s//%d %s %s %s %s\n",
            key,
            trace.ncall,
            trace.wt.to_i.to_s,
            trace.cpu.to_i.to_s,
            trace.al.to_i.to_s,
            trace.pmu.to_i.to_s
          )
      end

      @output << metrics
    end
  end
end
