# rubocop:disable Style/ClassVars
require "patches/sql_patches"

module Rack
  class BlackfireRack
    def initialize(app, _options = {})
      SqlPatches.patch

      @app = app
      @@current_path = ""
      @@current_percent = 0
    end

    def call(env)
      request = Rack::Request.new(env)

      if request.env.include?("HTTP_X_BLACKFIRE_QUERY") && should_profile?(request.path)
        if request.path != @@current_path
          @@current_path = request.path
          @@current_percent = 0
        end

        @@current_header = ""
        ::Blackfire.enable(request.env["HTTP_X_BLACKFIRE_QUERY"])
        begin
          result = @app.call(env)
        end
        ::Blackfire.end
        result[1]["X-Blackfire-Response"] = @@current_header unless @@current_header.empty?

        result
      else
        @app.call(env)
      end
    end

    def self.define_header(header)
      @@current_header = if header.include?("Blackfire-Response:")
                           ret = if header.include?("\n")
                                   header.split("\n")[0]
                                 else
                                   head
                                 end
                           ret.split("Blackfire-Response:")[1]
                         else
                           ""
                         end
    end

    private

    def should_profile?(path)
      return false if paths_match?(path, [%r{^/assets}, /\.(css|js|png|jpeg|jpg|gif)$/])

      true
    end

    def paths_match?(path, paths)
      paths.any? { |skip_path| skip_path =~ path }
    end
  end
end
# rubocop:enable Style/ClassVars
