# frozen_string_literal: true

module Rack
  class BlackfireRack
    class PG::Result
      alias each_without_profiling each
      alias values_without_profiling values

      def values(*args, &blk)
        return values_without_profiling(*args, &blk) unless defined?(@blackfire_sql_id)

        report_sql do
          values_without_profiling(*args, &blk)
        end
      end

      def each(*args, &blk)
        return each_without_profiling(*args, &blk) unless defined?(@blackfire_sql_id)

        report_sql do
          each_without_profiling(*args, &blk)
        end
      end

      def report_sql
        result = yield
        result
      end
    end

    class PG::Connection
      alias exec_without_profiling exec
      alias async_exec_without_profiling async_exec
      alias exec_prepared_without_profiling exec_prepared
      alias send_query_prepared_without_profiling send_query_prepared
      alias prepare_without_profiling prepare

      alias exec_params_without_profiling exec_params if Gem::Version.new(PG::VERSION) >= Gem::Version.new("1.1.0")

      def prepare(*args, &blk)
        @prepare_map ||= {}
        @prepare_map[args[0]] = args[1]
        @prepare_map = {} if @prepare_map.length > 1000

        return prepare_without_profiling(*args, &blk) unless SqlPatches.should_measure?

        Blackfire::QueryTrace.record_prepared_query(args[0], args[1])
        prepare_without_profiling(*args, &blk)
      end

      def exec(*args, &blk)
        return exec_without_profiling(*args, &blk) unless SqlPatches.should_measure?
        exec_without_profiling(*args, &blk)
      end

      if Gem::Version.new(PG::VERSION) >= Gem::Version.new("1.1.0")
        def exec_params(*args, &blk)
          return exec_params_without_profiling(*args, &blk) unless SqlPatches.should_measure?
          Blackfire::QueryTrace.record_sql(args[0], nil)
          exec_params_without_profiling(*args, &blk)
        end
      end

      def exec_prepared(*args, &blk)
        return exec_prepared_without_profiling(*args, &blk) unless SqlPatches.should_measure?
        Blackfire::QueryTrace.record_sql(args[0], nil)
        exec_prepared_without_profiling(*args, &blk)
      end

      def send_query_prepared(*args, &blk)
        return send_query_prepared_without_profiling(*args, &blk) unless SqlPatches.should_measure?
        send_query_prepared_without_profiling(*args, &blk)
      end

      def async_exec(*args, &blk)
        return async_exec_without_profiling(*args, &blk) unless SqlPatches.should_measure?
        exec_without_profiling(*args, &blk)
      end

      alias query exec
    end
  end
end
