# Disable minitest parallel tests. The problem is the thread switching will cahnge test results
# (self vs wait time)
ENV["MT_CPU"] = "0"

require "bundler/setup"
require "minitest/autorun"
require "minitest/fail_fast"
require "minitest/reporters"
require "blackfire"

Minitest::Reporters.use! Minitest::Reporters::SpecReporter.new

class TestCase < Minitest::Test
  # rubocop:disable Style/ClassVars
  @@current_server_id = ""
  @@current_server_token = ""
  @@original_server_id_was_set = false

  def setup
    return if @@original_server_id_was_set

    @@current_server_id = ENV["BLACKFIRE_SERVER_ID"]
    @@current_server_token = ENV["BLACKFIRE_SERVER_TOKEN"]
    ENV["BLACKFIRE_SERVER_ID"] = "977fbc63-8e3c-40f9-8c9c-157d2ac1a09e"
    ENV["BLACKFIRE_SERVER_TOKEN"] = "servertoken"
    @@original_server_id_was_set = true
  end

  def teardown
    return unless @@original_server_id_was_set

    ENV["BLACKFIRE_SERVER_ID"] = @@current_server_id
    ENV["BLACKFIRE_SERVER_TOKEN"] = @@current_server_token
    @@original_server_id_was_set = false
  end
  # rubocop:enable Style/ClassVars

  def build_server
    server = TCPServer.new("127.0.0.1", 0)
    port = server.addr[1]
    blackfire_socket = "tcp://127.0.0.1:#{port}".freeze

    [server, blackfire_socket]
  end

  def cleanup_env_vars
    ENV["BLACKFIRE_QUERY"] = nil
    ENV["BLACKFIRE_AGENT_SOCKET"] = nil
  end

  def burn_cpu(secs)
    t0 = Time.now
    loop do
      (0..100).each do |i|
        # noop
      end
      break if Time.now - t0 >= secs
    end
    #puts format("Elapsed=%0.6f", Time.now - t0)
  end

  def configure_env_vars(timespan: false, no_agent_ids: false, socket: "")
    bfq_timespan = "expires=1591993676&userId=a5a46fc7-4f91-40a5-a7c8-d784048b1d60&" \
                   "agentIds=977fbc63-8e3c-40f9-8c9c-157d2ac1a09e%2C0f78ecba-b580-437d-b502-82bc9346a021%2Ca38f69ca-3b56-4ea6-beac-272b2568164a%2C-%" \
                   "2Crequest-id-29d32615-aee1-43d0-afdc-a386afbe4160&profileSlot=00000000-0000-0000-0000-000000000000&" \
                   "collabToken=5a670229-632d-42e1-971b-1e618e07d565" \
                   "&signature=RWSfcPNpzVzuR7CRaOGZvY0MNGLkkCtGwPEmECKSmy8_Xf8-JLnqFZ66EXuxiTv6q0ys-C2kG9s9K6dvWr9wg2Ebd-dUb21gfQY%3D" \
                   "&aggreg_samples=1&flag_composer=1&flag_cpu=1&flag_fn_args=1&flag_memory=1&flag_nw=1&flag_pdo=1&flag_sessions=1&" \
                   "flag_timespan=1&flag_yml=1&no_anon=0&no_pruning=0&timespan_threshold=10&timespan_threshold_max=100&" +
                   "timespan_threshold_min=1&timespan_threshold_ratio=0.01".freeze

    bfq_no_agent_ids = "expires=1591993676&userId=a5a46fc7-4f91-40a5-a7c8-d784048b1d60&" \
                       "agentIds=0f78ecba-b580-437d-b502-82bc9346a021%2Ca38f69ca-3b56-4ea6-beac-272b2568164a%2C-%" \
                       "2Crequest-id-29d32615-aee1-43d0-afdc-a386afbe4160&profileSlot=00000000-0000-0000-0000-000000000000&" \
                       "collabToken=5a670229-632d-42e1-971b-1e618e07d565" \
                       "&signature=RWSfcPNpzVzuR7CRaOGZvY0MNGLkkCtGwPEmECKSmy8_Xf8-JLnqFZ66EXuxiTv6q0ys-C2kG9s9K6dvWr9wg2Ebd-dUb21gfQY%3D" \
                       "&aggreg_samples=1&flag_composer=1&flag_cpu=1&flag_fn_args=1&flag_memory=1&flag_nw=1&flag_pdo=1&flag_sessions=1&" \
                       "flag_timespan=0&flag_yml=1&no_anon=0&no_pruning=0&timespan_threshold=10&timespan_threshold_max=100&" +
                       "timespan_threshold_min=1&timespan_threshold_ratio=0.01".freeze

    bfq_no_timespan = "expires=1591993676&userId=a5a46fc7-4f91-40a5-a7c8-d784048b1d60&" \
                      "agentIds=977fbc63-8e3c-40f9-8c9c-157d2ac1a09e%2C0f78ecba-b580-437d-b502-82bc9346a021%2Ca38f69ca-3b56-4ea6-beac-272b2568164a%2C-%" \
                      "2Crequest-id-29d32615-aee1-43d0-afdc-a386afbe4160&profileSlot=00000000-0000-0000-0000-000000000000&" \
                      "collabToken=5a670229-632d-42e1-971b-1e618e07d565" \
                      "&signature=RWSfcPNpzVzuR7CRaOGZvY0MNGLkkCtGwPEmECKSmy8_Xf8-JLnqFZ66EXuxiTv6q0ys-C2kG9s9K6dvWr9wg2Ebd-dUb21gfQY%3D" \
                      "&aggreg_samples=1&flag_composer=1&flag_cpu=1&flag_fn_args=1&flag_memory=1&flag_nw=1&flag_pdo=1&flag_sessions=1&" \
                      "flag_timespan=0&flag_yml=1&no_anon=0&no_pruning=0&timespan_threshold=10&timespan_threshold_max=100&" +
                      "timespan_threshold_min=1&timespan_threshold_ratio=0.01".freeze

    ENV["BLACKFIRE_AGENT_SOCKET"] = socket
    ENV["BLACKFIRE_QUERY"] = bfq_timespan if timespan && !no_agent_ids
    ENV["BLACKFIRE_QUERY"] = bfq_no_timespan if !timespan && !no_agent_ids
    ENV["BLACKFIRE_QUERY"] = bfq_no_agent_ids if !timespan && no_agent_ids
  end

  # rubocop:disable Metrics/CyclomaticComplexity
  # rubocop:disable Metrics/PerceivedComplexity
  def ignore_until_threshold_starts(client)
    ret = client.gets

    loop do
      break if ret.nil?
      break if ret == "\n"

      ret = client.gets
    end

    client.puts "Blackfire-Response: continue=false&first_sample=true&blackfire_yml=true"
    client.puts "Blackfire-Keys: 2884905623;RWSfcPNpzVzuR1cRAkMPmQT9R6z4uHY0d8reTmCKcnaAHmu1csm9jLxZ"
    client.puts ""

    loop do
      ret = client.gets

      break if ret.nil?
      break if ret.include?("Threshold")
      break if ret == "\n"

      if ret.start_with? "Blackfire-Yaml-Size: "
        bytes = ret.split(" ")[1].to_i
        client.read(bytes) if bytes.positive?
      end
    end
    ret
  end
  # rubocop:enable Metrics/CyclomaticComplexity
  # rubocop:enable Metrics/PerceivedComplexity

  def ignore_until_threshold_ends(client)
    ret = ignore_until_threshold_starts(client)

    loop do
      break unless ret.include?("Threshold")

      ret = client.gets
    end
    ret
  end

  def ignore_until_empty_line(client)
    ret = client.gets
    loop do
      break if ret.nil?
      break if ret == "\n"

      ret = client.gets
    end
    ret
  end
end
